"""Top-level package for Auto deprecator."""
# flake8: noqa

__author__ = """Gavin Chan"""
__email__ = "gavincyi@gmail.com"
__version__ = "2020.2.0"

from .deprecate import (
    deprecate,
    check_deprecation,
)
